"""
Configuration provider package, that exposes the section as attribute and also automatically keeps the config file and object in sync.
Through the Factory/Strategy the config objects are singletons as long as they point to the same file.
"""

__version__ = '0.2.7'

from .standard import *
from .data import *
import logging
import gidlogger as glog

log = glog.library_base_logger(__name__)





# region [Imports]


import asyncio
import gc
import os
import re
import sys
import json
import lzma
import time
import queue
import platform
import subprocess
from enum import Enum, Flag, auto
from time import sleep, time
from pprint import pprint, pformat
from typing import Union
from datetime import tzinfo, datetime, timezone, timedelta
from functools import wraps, lru_cache, singledispatch, total_ordering, partial
from contextlib import contextmanager
from collections import Counter, ChainMap, deque, namedtuple, defaultdict
from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
from inspect import isfunction, ismethod, isroutine, isclass


import gidlogger as glog


# endregion[Imports]

# region [TODO]


# endregion [TODO]

# region [AppUserData]


# endregion [AppUserData]

# region [Logging]

log = glog.aux_library_logger(__name__)

# endregion[Logging]

# region [Constants]


# endregion[Constants]


def debug_timing_print(func):
    @wraps(func)
    def _function_print_time(*args, **kwargs):
        start_time = time()
        _out = func(*args, **kwargs)
        if hasattr(args[0], func.__name__):
            report = f"'{func.__name__}' of the '{args[0].__class__.__name__}' class took {str(round(time()-start_time, ndigits=4))} seconds"
        else:
            report = f"'{func.__name__}' took {str(round(time()-start_time, ndigits=4))} seconds"

        print(report)
        return _out
    if os.getenv('IS_DEV') == 'true':
        return _function_print_time
    else:
        return func


def debug_timing_log(logger):
    def _decorator(func):
        @wraps(func)
        def _function_print_time(*args, **kwargs):
            start_time = time()
            _out = func(*args, **kwargs)
            if hasattr(args[0], func.__name__):
                report = f"'{func.__name__}' of '{str(args[0])}' took {str(round(time()-start_time, ndigits=4))} seconds"
            else:
                report = f"'{func.__name__}' took {str(round(time()-start_time, ndigits=4))} seconds"

            logger.debug(report, extra={'func_name_override': func.__name__})
            return _out
        if os.getenv('IS_DEV') == 'true':
            return _function_print_time
        else:
            return func

    return _decorator


# region[Main_Exec]


if __name__ == '__main__':
    pass

# endregion[Main_Exec]



# region [Imports]

# * Standard Library Imports -->
from functools import partial

# * Gid Imports -->
import gidlogger as glog

# * Local Imports -->
from gidconfig.utility.functions import loadjson
from gidconfig.experimental.experimental_abstract import GidAttConfigAbstract

# endregion[Imports]

__updated__ = '2020-11-14 16:05:22'

# region [AppUserData]

# endregion [AppUserData]

# region [Logging]

log = glog.aux_library_logger(__name__)
log.info(glog.imported(__name__))

# endregion[Logging]

# region [Constants]

# endregion[Constants]


class GidAttConfigJson(GidAttConfigAbstract):
    def __init__(self, config_file):
        super().__init__(config_file)

    def load(self):
        _config_dict = loadjson(self.config_file)
        for section, value in _config_dict.items():
            setattr(self, section, value)
            self.added_attributes.append(section)
            setattr(self, 'set_' + section, partial(self._edit_dict_attribute, section))
            self.added_attributes.append('set_' + section)


# region[Main_Exec]

if __name__ == '__main__':
    pass

# endregion[Main_Exec]



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['gidconfig',
 'gidconfig.data',
 'gidconfig.experimental',
 'gidconfig.standard',
 'gidconfig.utility']

package_data = \
{'': ['*']}

install_requires = \
['fuzzywuzzy==0.18.0', 'gidlogger>=0.1.33']

setup(name='gidconfig',
      version='0.2.7',
      description='Configuration provider package, that exposes the section as attribute and also automatically keeps the config file and object in sync.',
      author='Giddius',
      author_email=None,
      url='https://github.com/Giddius/Gidconfig',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )



from .experimental_ini import GidAttConfigIni
from .experimental_json import GidAttConfigJson
from .experimental_abstract import GidAttConfigAbstract
from .experimental_strategy import ConfigStrategos



# region [Imports]

# * Gid Imports -->
import gidlogger as glog

# endregion [Imports]

# region [Logging]

log = glog.aux_library_logger(__name__)
log.info(glog.imported(__name__))

# endregion [Logging]


# region [Main_Exec]
if __name__ == '__main__':
    pass


# endregion [Main_Exec]
